'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _Parser = require('gremlin-console/lib/Parser');

var _Parser2 = _interopRequireDefault(_Parser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Client that communicates via ajax
 *
 * @author Dylan Millikin <dylan.millikin@gmail.com>
 */

var AjaxClient = function () {

    /**
     * Create the client
     *
     * @param  {String}  host    the host name / ip
     * @param  {Integer} port    the port number for the client to connect to
     * @param  {Object}  options the driver options as per defined in the driver documentation
     * @return void
     */


    /**
     * @type {Parser} a Parser object used to parse the data
     */

    function AjaxClient(host, port, options, parser) {
        _classCallCheck(this, AjaxClient);

        if (typeof parser === "undefined") {
            this.parser = new _Parser2.default();
        } else {
            this.parser = parser;
        }

        var path = options.path || "";
        delete options.path;

        this._options = _extends({
            type: "POST",
            dataType: "text",
            url: host + ':' + port + '/' + path.replace(/^(\/)/, ""),
            data: {
                session: this._createUUID()
            },
            error: function error(err) {
                console.log(err);
            }
        }, options);
    }

    /**
     * Run a query with various params.
     * Bellow are the three expected params. optionals can be ommitted and interchanged
     *
     * @param  {String}   query    mandatory: the gremlin query to run
     * @param  {Object}   bindings optional: the bindings associated to this query
     * @param  {Function} callback optional: function that executes once the results are received.
     * @return {Void}
     */


    /**
     * @type {Object} an object containing all the configuration for the ajax call
     */


    _createClass(AjaxClient, [{
        key: 'execute',
        value: function execute(query, bindings, callback) {
            var _this = this;

            if (typeof bindings === 'function') {
                callback = bindings;
                bindings = {};
            }

            //customize the callback params to use Parser
            this._options.success = function (data) {
                data = JSON.parse(data);
                callback(_this.parser.create(data.err, data.results));
            };

            this._options.data.gremlin = query;
            this._options.data.bindings = bindings;

            _jquery2.default.ajax(this._options);
        }

        /**
         * Register a callback on error
         * It's rare to need this but sometimes you'll want to register a sepcific behavior against the client's error management
         *
         * @param  {Function} callback the method to run on client error.
         * @return {Void}
         */

    }, {
        key: 'onError',
        value: function onError(callback) {
            var customCallback = function customCallback(j, s, e) {
                callback({ status: s, message: e });
            };
            this._options.error = customCallback;
        }

        /**
         * Register a callback on open
         *
         * @param  {Function} callback the method to run on client error.
         * @return {Void}
         */

    }, {
        key: 'onOpen',
        value: function onOpen(callback) {
            callback();
        }

        /**
         * Generates a UUID
         *
         * return {String} the UUID for this client
         */

    }, {
        key: '_createUUID',
        value: function _createUUID() {
            var d = new Date().getTime();
            if (window.performance && typeof window.performance.now === "function") {
                d += performance.now(); //use high-precision timer if available
            }
            var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                var r = (d + Math.random() * 16) % 16 | 0;
                d = Math.floor(d / 16);
                return (c == 'x' ? r : r & 0x3 | 0x8).toString(16);
            });
            return uuid;
        }
    }]);

    return AjaxClient;
}();

exports.default = AjaxClient;